/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.xwiki.edit.test.ui;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.xwiki.edit.test.po.InplaceEditablePage;
import org.xwiki.test.docker.junit5.TestReference;
import org.xwiki.test.docker.junit5.UITest;
import org.xwiki.test.ui.TestUtils;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * Tests in-place page editing.
 * 
 * @version $Id: 8128f87388c1e4ee36c2807a27c4ffecbb42ba47 $
 * @since 12.10.6
 * @since 13.2RC1
 */
@UITest
class InplaceEditIT
{
    @BeforeEach
    void setup(TestUtils setup, TestReference testReference)
    {
        setup.createUserAndLogin("alice", "pa$$word", "editor", "Wysiwyg");
        setup.createPage(testReference, "before\n\n== Section ==\n\nafter", "test title");
    }

    @Test
    void editInplace(TestUtils setup, TestReference testReference)
    {
        InplaceEditablePage viewPage = new InplaceEditablePage();
        assertTrue(viewPage.getPageURL().endsWith("/editInplace/"), viewPage.getPageURL());

        //
        // Test action buttons (Cancel, Save, Save & View).
        //

        // Cancel
        viewPage.editInplace();
        assertEquals("test title", viewPage.getDocumentTitle());
        assertTrue(viewPage.getPageURL().endsWith("/editInplace/#edit"), viewPage.getPageURL());

        viewPage.setDocumentTitle("updated title");
        viewPage.cancel();
        assertEquals("test title", viewPage.getDocumentTitle());
        assertTrue(viewPage.getPageURL().endsWith("/editInplace/#"), viewPage.getPageURL());

        // Save + Cancel
        viewPage.editInplace();
        assertEquals("test title", viewPage.getDocumentTitle());
        assertTrue(viewPage.getPageURL().endsWith("/editInplace/#edit"), viewPage.getPageURL());

        viewPage.setDocumentTitle("updated title");
        viewPage.save();
        viewPage.setDocumentTitle("new title");
        viewPage.cancel();
        assertEquals("updated title", viewPage.getDocumentTitle());

        // Save & View
        viewPage.editInplace();
        assertEquals("updated title", viewPage.getDocumentTitle());

        viewPage.setDocumentTitle("new title");
        viewPage.saveAndView();
        assertEquals("new title", viewPage.getDocumentTitle());

        // Save + Save & View
        viewPage.editInplace();
        assertEquals("new title", viewPage.getDocumentTitle());

        viewPage.setDocumentTitle("another title");
        viewPage.save();
        viewPage.setDocumentTitle("last title");
        viewPage.saveAndView();
        assertEquals("last title", viewPage.getDocumentTitle());

        //
        // Test section editing
        //

        viewPage.editSectionInplace(1);
        assertEquals("last title", viewPage.getDocumentTitle());

        viewPage.setDocumentTitle("Last Title");
        viewPage.save().cancel();
        assertEquals("Last Title", viewPage.getDocumentTitle());

        //
        // Test in-place editor reload.
        //

        viewPage.editInplace();
        viewPage.setDocumentTitle("some title");
        setup.getDriver().navigate().refresh();
        viewPage = new InplaceEditablePage().waitForInplaceEditor();
        // Changes are currently lost on page reload.
        assertEquals("Last Title", viewPage.getDocumentTitle());

        viewPage.setDocumentTitle("final title");
        viewPage.saveAndView();
        assertEquals("final title", viewPage.getDocumentTitle());
    }
}
